@@ -5,6 +5,7 @@ import inspect
 from abc import ABC, abstractmethod
 
 import pandas as pd
+import numpy as np
 
 
 def _get_tags(self):
@@ -24,7 +25,9 @@ def _get_tags(self):
         "multioutput_only": False,
         "binary_only": False,
         "requires_fit": True,
+        "preserves_dtype": [np.float64],
         "requires_y": False,
+        "pairwise": False,
     }
 
 
@@ -128,6 +131,8 @@ class DiscovererMixin:
         pd.Series
             patterns discovered by a mining algorithm
         """
+        if y is None:
+            return self.fit(D).discover(**kwargs)
         return self.fit(D, y=y).discover(**kwargs)
 
 
