@@ -4,6 +4,7 @@ Base IO code for all datasets
 
 import os
 
+
 def get_data_home(data_home=None):
     """Return the path of the scikit-mine data home directory.
 
@@ -21,8 +22,9 @@ def get_data_home(data_home=None):
         The path to scikit-mine data dir.
     """
     if data_home is None:
-        data_home = os.environ.get('SCIKIT_MINE_DATA',
-                                   os.path.join('~', 'scikit_mine_data'))
+        data_home = os.environ.get(
+            "SCIKIT_MINE_DATA", os.path.join("~", "scikit_mine_data")
+        )
     data_home = os.path.expanduser(data_home)
     if not os.path.exists(data_home):
         os.makedirs(data_home)
