@@ -3,7 +3,8 @@
 # Authors: Rémi Adon <remi.adon@gmail.com>
 # License: BSD 3 clause
 
-from functools import reduce, lru_cache
+from collections import defaultdict
+from functools import lru_cache, reduce
 from itertools import chain
 
 import numpy as np
@@ -12,9 +13,15 @@ from sortedcontainers import SortedDict
 
 from ..base import BaseMiner, MDLOptimizer
 from ..bitmaps import Bitmap
-from ..utils import supervised_to_unsupervised
-from ..utils import _check_D
-from ..callbacks import mdl_prints
+from ..utils import _check_D, supervised_to_unsupervised
+
+
+def _to_vertical(D):
+    res = defaultdict(Bitmap)
+    for idx, transaction in enumerate(D):
+        for e in transaction:
+            res[e].add(idx)
+    return dict(res)
 
 
 def _log2(values):
@@ -24,15 +31,15 @@ def _log2(values):
     return pd.Series(res, index=res_index)
 
 
-def cover(sct: SortedDict, itemsets: list):
+def cover(sct: dict, itemsets: list):
     """
     cover a standard codetable sct given itemsets
 
     Parameters
     ----------
-    sct: SortedDict[object, Bitmap]
+    sct: dict[object, Bitmap]
         a standard codetable, i.e the vertical representation of a dataset
-    itemsets: list
+    itemsets: list[frozenset]
         itemsets from a given codetable
 
     Notes
@@ -53,56 +60,17 @@ def cover(sct: SortedDict, itemsets: list):
     return covers
 
 
-def update_usages(sct, ct, candidate, delete=False):
-    """
-    Parameters
-    ----------
-    sct: SortedDict[frozenset, Bitmap]
-        a standard codetable, i.e the vertical representation of a dataset
+def reconstruct(codetable, n_transactions=None):
+    """reconstruct the original data from the `codetable`"""
+    if n_transactions is None:
+        n_transactions = (
+            max(map(Bitmap.max, filter(lambda e: e, codetable.values()))) + 1
+        )
 
-    ct: SortedDict[frozenset, Bitmap]
-        A codetable, sorted in Standard Candidate Order
-
-    candidate: object
-        candidate to consider for update
-
-    delete: bool, default=False
-        either to delete or add `candidate` in ct
-
-    Returns
-    -------
-    dict, dict
-        a tuple, first dict is containing the updated usage,
-        seconde dict contains decreased usages (for later pruning)
-    """
-    new_isets = list(ct.keys())
-
-    if delete:
-        new_isets.remove(candidate)
-    else:
-        cand_pos = ct.bisect(candidate)
-        new_isets.insert(cand_pos, candidate)
-
-    _sct = {k: tids.copy() for k, tids in sct.items()}
-    update_d = dict()
-    for iset in new_isets:
-        it = (_sct[i] for i in iset)
-        usage = reduce(Bitmap.intersection, it)
-        if iset == candidate:  # candidate should never be a singleton
-            update_d[iset] = usage
-        elif usage != ct[iset]:
-            usage = usage.copy()
-            update_d[iset] = usage
-        if usage:
-            for k in iset:
-                _sct[k] -= usage
-
-    # assert not _sct
-    decreased = {
-        k for k, v in update_d.items() if k != candidate and len(v) < len(ct[k])
-    }
-
-    return update_d, decreased
+    D = pd.Series([set()] * n_transactions)
+    for itemset, tids in codetable.items():
+        D.iloc[list(tids)] = D.iloc[list(tids)].map(itemset.union)
+    return D.map(sorted)
 
 
 def generate_candidates_big(codetable, stack=None):
@@ -179,25 +147,21 @@ class SLIM(BaseMiner, MDLOptimizer):
         Number of candidate evaluation with no improvement to count before stopping optimization.
     tol: float, default=None
         Tolerance for the early stopping, in bits.
-        When the compression size is not improving by at least tol for n_iter_no_change iterations,
-        the training stops.
+        When the compression size is not improving by at least `tol` for `n_iter_no_change`
+        iterations, the training stops.
         Default to None, will be automatically computed considering the size of input data.
-    pruning: bool, default=False
+    pruning: bool, default=True
         Either to activate pruning or not. Pruned itemsets may be useful at
         prediction time, so it is usually recommended to set it to False
         to build a classifier. The model will be less concise, but will lead
         to more accurate predictions on average.
-    verbose: integer
-        Controls the verbosity: the higher, the more messages.
 
 
     Examples
     --------
     >>> from skmine.itemsets import SLIM
-    >>> from skmine.preprocessing import TransactionEncoder
     >>> D = [['bananas', 'milk'], ['milk', 'bananas', 'cookies'], ['cookies', 'butter', 'tea']]
-    >>> D = TransactionEncoder().fit_transform(D)
-    >>> SLIM().fit(D)                       # doctest: +SKIP
+    >>> SLIM().fit(D).codetable  # doctest: +SKIP
     (butter, tea)         [2]
     (milk, bananas)    [0, 1]
     (cookies)          [1, 2]
@@ -213,34 +177,28 @@ class SLIM(BaseMiner, MDLOptimizer):
         "Slimmer, outsmarting Slim", 2014
     """
 
-    def __init__(self, *, n_iter_no_change=100, tol=None, pruning=False, verbose=False):
+    def __init__(self, *, n_iter_no_change=100, tol=None, pruning=True):
         self.n_iter_no_change = n_iter_no_change
         self.tol = tol
         self.standard_codetable_ = None
-        self.codetable_ = pd.Series([], dtype="object")
+        self.codetable_ = SortedDict()
         self.model_size_ = None  # L(CT|D)
         self.data_size_ = None  # L(D|CT)
         self.pruning = pruning
-        self.verbose = verbose
-
-        mdl_prints(self)  # attach mdl_prints <-- output if self.verbose set
 
     def fit(self, D, y=None):  # pylint:disable = too-many-locals
         """fit SLIM on a transactional dataset
 
         This generate new candidate patterns and add those which improve compression,
-        iteratibely refining ``self.codetable``
+        iteratibely refining ``self.codetable_``
 
         Parameters
         -------
         D: pd.DataFrame
-            Transactional dataset, encoded as tabular binary data
+            Transactional dataset, either as an iterable of iterables
+            or encoded as tabular binary data
         """
-        D = _check_D(D)
-        if y is not None:
-            D = supervised_to_unsupervised(D, y)  # SKLEARN_COMPAT
-
-        self._prefit(D)
+        self._prefit(D, y=y)
         n_iter_no_change = 0
         seen_cands = set()
 
@@ -264,8 +222,6 @@ class SLIM(BaseMiner, MDLOptimizer):
 
                 if diff < tol:
                     n_iter_no_change += 1
-                    if self.verbose:
-                        print("n_iter_no_change : {}".format(n_iter_no_change))
                     if n_iter_no_change > self.n_iter_no_change:
                         break  # inner break
 
@@ -277,27 +233,35 @@ class SLIM(BaseMiner, MDLOptimizer):
     def decision_function(self, D):
         """Compute covers on new data, and return code length
 
-        This function function is named ``decision_funciton`` because code lengths
+        This function function is named ``decision_function`` because code lengths
         represent the distance between a point and the current codetable.
 
         Setting ``pruning`` to False when creating the model
         is recommended to cover unseen data, and especially when building a classifier.
 
+        Parameters
+        ----------
+        D: pd.DataFrame or np.ndarray
+            new data to make predictions on, in tabular format
+
         Example
         -------
-        >>> from skmine.preprocessing import TransactionEncoder
+        >>> from skmine.itemsets import SLIM; import pandas as pd
+        >>> def to_tabular(D): return pd.Series(D).str.join('|').str.get_dummies(sep="|")
         >>> D = [['bananas', 'milk'], ['milk', 'bananas', 'cookies'], ['cookies', 'butter', 'tea']]
-        >>> te = TransactionEncoder()
-        >>> D = te.fit_transform(D)
-        >>> new_D = te.transform([['cookies', 'butter']])
-        >>> slim = SLIM().fit(D)
+        >>> new_D = to_tabular([['cookies', 'butter']])
+        >>> slim = SLIM().fit(to_tabular(D))
         >>> slim.decision_function(new_D)
         0   -1.321928
         dtype: float32
         """
         D = _check_D(D)
-        codetable = self.codetable  # this is a function call, beware
-        D_sct = {k: Bitmap(np.where(D[k])[0]) for k in D.columns}
+        codetable = pd.Series(self.codetable_)
+        D_sct = {
+            k: Bitmap(np.where(D[k])[0])
+            for k in D.columns
+            if k in self.standard_codetable_
+        }
         covers = cover(D_sct, codetable.index)
 
         mat = np.zeros(shape=(len(D), len(covers)))
@@ -308,8 +272,10 @@ class SLIM(BaseMiner, MDLOptimizer):
         code_lengths = codetable.map(len)
         ct_codes = code_lengths / code_lengths.sum()
         codes = (mat * ct_codes).sum(axis=1).astype(np.float32)
-        # positive sign on log2 to return negative distance : sklearn
-        return _log2(codes)
+        # positive sign on log2 to return negative distance : sklearn]
+        r = _log2(codes)
+        r[r == 0] = -np.inf  # zeros would fool a `shortest code wins` strategy
+        return r
 
     def generate_candidates(self, stack=None, thresh=1e3):
         """
@@ -350,26 +316,26 @@ class SLIM(BaseMiner, MDLOptimizer):
             updated (data size, model size, codetable)
             and finally the set of itemsets for which usage decreased
         """
-        update_d, decreased = update_usages(
-            self.standard_codetable_, self.codetable_, candidate
-        )
+        idx = self.codetable_.bisect(candidate)
+        ct = list(self.codetable_)
+        ct.insert(idx, candidate)
+        D = {k: v.copy() for k, v in self.standard_codetable_.items()}
+        CTc = cover(D, ct)
 
-        CTc = {**self.codetable_, **update_d}
+        updated, decreased = {candidate: CTc[candidate]}, set()
+        for iset, usage in self.codetable_.items():  # TODO useless is size is too big
+            if usage != CTc[iset]:
+                updated[iset] = CTc[iset]
+                if len(CTc[iset]) < len(usage):
+                    decreased.add(iset)
 
-        data_size, model_size = self._compute_sizes(CTc)
+        data_size, model_size = self._compute_sizes(CTc)  # TODO pruning in evaluate
 
-        return data_size, model_size, update_d, decreased
+        return data_size, model_size, updated, decreased
 
     def reconstruct(self):
-        """reconstruct the original data from the current codetable"""
-        ct = self.codetable
-        n_transactions = ct.map(Bitmap.max).max() + 1
-
-        D = pd.Series([set()] * n_transactions)
-
-        for itemset, tids in ct.iteritems():
-            D.iloc[list(tids)] = D.iloc[list(tids)].map(itemset.union)
-        return D.map(sorted)
+        """reconstruct the original data from the current `self.codetable_`"""
+        return reconstruct(self.codetable_)
 
     @lru_cache(maxsize=1024)
     def get_support(self, itemset):
@@ -387,8 +353,14 @@ class SLIM(BaseMiner, MDLOptimizer):
     def _standard_candidate_order(self, itemset):
         return (-self.get_support(itemset), -len(itemset), tuple(itemset))
 
-    def _prefit(self, D):
-        item_to_tids = {k: Bitmap(np.where(D[k])[0]) for k in D.columns}
+    def _prefit(self, D, y=None):
+        if hasattr(D, "ndim") and D.ndim == 2:
+            D = _check_D(D)
+            if y is not None:
+                D = supervised_to_unsupervised(D, y)  # SKLEARN_COMPAT
+            item_to_tids = {k: Bitmap(np.where(D[k])[0]) for k in D.columns}
+        else:
+            item_to_tids = _to_vertical(D)
         self.standard_codetable_ = pd.Series(item_to_tids)
         usage = self.standard_codetable_.map(len).astype(np.uint32)
 
@@ -448,10 +420,9 @@ class SLIM(BaseMiner, MDLOptimizer):
 
         Parameters
         ----------
-        codetable: pd.Series
-        D: pd.Series
-        prune_set: pd.Series
-            subset of ``codetable`` for which usage has decreased
+        codetable: SortedDict
+        prune_set: set
+            itemsets in ``codetable`` for which usage has decreased
         model_size: float
             current model_size for ``codetable``
         data_size: float
@@ -468,17 +439,21 @@ class SLIM(BaseMiner, MDLOptimizer):
             cand = min(prune_set, key=lambda e: len(codetable[e]))
             prune_set.discard(cand)
 
-            update_d, decreased = update_usages(
-                self.standard_codetable_, codetable, cand, delete=True
-            )
+            ct = list(codetable)
+            ct.remove(cand)
 
-            CTp = {**codetable, **update_d}
-            del CTp[cand]
+            D = {
+                k: v.copy() for k, v in self.standard_codetable_.items()
+            }  # TODO avoid data copies
+            CTp = cover(D, ct)
+            decreased = {
+                k for k, v in CTp.items() if len(k) > 1 and len(v) < len(codetable[k])
+            }
 
             d_size, m_size = self._compute_sizes(CTp)
 
             if d_size + m_size < model_size + data_size:
-                codetable.update(update_d)
+                codetable.update(CTp)
                 del codetable[cand]
                 prune_set.update(decreased)
                 data_size, model_size = d_size, m_size
